﻿using System;
using System.Diagnostics;
using System.ServiceModel;
using System.ServiceModel.Description;
using System.ServiceProcess;
using System.Threading;
using TpiService.Tpi;
using WcfJsonRestService;

namespace TpiService
{
    partial class AlarmService : ServiceBase
    {
        private ServiceHost _host;
        
        private const string LogSource = "Alarm";
        private const string LogName = "AlarmLog";

        private static CommunicationInitor _communicationInitor;

        public AlarmService()
        {
            InitializeComponent();
            if (!EventLog.SourceExists(LogSource))
            {
                EventLog.CreateEventSource(LogSource, LogName);
            }
            eventLog.Source = LogSource;
            eventLog.Log = LogName;
        }

        protected override void OnStart(string[] args)
        {
            eventLog.WriteEntry("Alarmservice starting");
            _communicationInitor = new CommunicationInitor(eventLog);
           //start jsonservice
            eventLog.WriteEntry("JSON Service starting");
            _host = new ServiceHost(typeof(JsonService));
            _host.Open();
        }

        protected override void OnStop()
        {
            eventLog.WriteEntry("Alarmservice stopping");
            _communicationInitor.Shutdown();
            eventLog.WriteEntry("JSON Service stopping");
            if (_host != null)
            {
                _host.Close();
            }
        }

        public static void ConfigChanged()
        {
            _communicationInitor.Shutdown();
            Thread.Sleep(500);
            _communicationInitor.Init();
        }
    }
}
