﻿using System;
using System.Configuration;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Reflection;
using Common;
using Common.Commands;
using Logic.Communications;

namespace TpiService.Tpi
{
    public class CommunicationInitor
    {
        private string _password;
        private string _server;
        private readonly EventLog _eventLog;


        public CommunicationInitor(EventLog eventLog)
        {
            _eventLog = eventLog;
            Init();
        }

        public void Init()
        {
            if (String.IsNullOrEmpty(ConfigurationManager.AppSettings["server"]) || String.IsNullOrEmpty(ConfigurationManager.AppSettings["password"]))
            {
                _eventLog.WriteEntry("server and/or password missing in app.settings");
                return;
            }

            //check if we have server and password.
            var v = new AppSettingsReader();
            var server = v.GetValue("server", typeof(string)) as string;
            var password = v.GetValue("password", typeof(string)) as string;
            if (server != null || password != null)
            {
                _password = password;
                _server = server;
            }
            Communicator.GetCommunicator(_server, _eventLog); // init

            Communicator.GetInstance().Subscribe(new LoginHandler(ContinueWithLogin, WrongPassword));


            //loop through dll's
            var directory = Path.GetDirectoryName(Assembly.GetExecutingAssembly().Location);
            if (directory != null)
            {

                foreach (var file in Directory.GetFiles(Path.Combine(directory, "plugins")))
                {
                    if (file.Contains("dll"))
                    {
                        _eventLog.WriteEntry("Found plugin: " + file);
                        Assembly a = Assembly.LoadFile(file);
                        var types = from t in a.GetTypes()
                                    where typeof(IResponseHandler).IsAssignableFrom(t)
                                    select t;

                        foreach (var t in types)
                        {
                            var handler = Activator.CreateInstance(t, BindingFlags.CreateInstance, null, null, null) as IResponseHandler;
                            Communicator.GetInstance().Subscribe(handler);
                        }
                    }

                }
            }
            try
            {
                Communicator.GetInstance().Connect();
            }
            catch (Exception e)
            {
                _eventLog.WriteEntry("Error connecting to server\r\n" + e.StackTrace);
                Shutdown();
            }
        }


        public void ContinueWithLogin()
        {
            Communicator.GetInstance().ExecuteRequest(new TpiRequest(RequestCommand.NetworkLogin, _password));
            Communicator.GetInstance().ExecuteRequest(new TpiRequest(RequestCommand.TimeStampControl, "1")); // enable timestamp
            Communicator.GetInstance().EnablePoll(); // start poll (keep alive) thread
            Communicator.GetInstance().ExecuteRequest(new TpiRequest(RequestCommand.StatusReport));
        }
        public void WrongPassword()
        {
            Shutdown();
        }

        public void Shutdown()
        {
            Communicator.GetInstance().Quit();
        }


        public void Log(string message)
        {
            _eventLog.WriteEntry(message);
        }
    }
}

