﻿using System.ServiceModel.Web;
using Common.Commands;
using Logic.Communications;

namespace WcfJsonRestService
{
    public class JsonService : IJsonService
    {
        [WebInvoke(Method = "GET",
                    ResponseFormat = WebMessageFormat.Json,
                    UriTemplate = "awayarm/{partition}")]
        public string AwayArm(string partition)
        {
            GetCommunicator().ExecuteRequest(new TpiRequest(RequestCommand.PartitionArmControl));
            return "awayarm";
        }

        [WebInvoke(Method = "GET",
                    ResponseFormat = WebMessageFormat.Json,
                    UriTemplate = "stayarm/{partition}")]
        public string StayArm(string partition)
        {
            GetCommunicator().ExecuteRequest(new TpiRequest(RequestCommand.PartitionArmControlStayArm));
            return "stayarm";
        }

        [WebInvoke(Method = "GET",
            ResponseFormat = WebMessageFormat.Json,
            UriTemplate = "disarm/{partition}/{code}")]
        public string DisArm(string partition, string code)
        {
            GetCommunicator().ExecuteRequest(new TpiRequest(RequestCommand.PartitionDisarmControl, partition + code));
            return "disarmed";
        }

        [WebInvoke(Method = "GET",
            ResponseFormat = WebMessageFormat.Json,
            UriTemplate = "status/{partition}")]
        public string IsArmed(string partition)
        {
            var response = GetCommunicator().ExecuteRequestAndWaitForResponse(
                RequestCommand.StatusReport,
                new[] { ResponseCommand.PartitionDisarmed, ResponseCommand.PartitionArmed },
                partition
                );

            if (response.Command == ResponseCommand.PartitionArmed)
            {
                if (response.Data.Substring(1) == "0") return "AwayArmed";
                if (response.Data.Substring(1) == "1") return "StayArmed";
            }
            if (response.Command == ResponseCommand.PartitionDisarmed)
            {
                return "Disarmed";
            }
            return "unknown";
        }

        public static Communicator GetCommunicator()
        {
            return Communicator.Instance;
        }
    }
}